import math
import ipdb
import astropy.units as u
import matplotlib.pyplot as plt
import numpy as np
from fnmatch import fnmatch
from astropy.io import ascii
from astropy.table import Table
from desk import config, fitting_tools
from matplotlib import rc


def get_model_and_data_for_plotting(counter, target):
    """Gets data from target.csv file and model from grid file.

    Parameters
    ----------
    counter : int
        The nth item being fit, starting at 1.
    target : astropy table row
        Results of fit item read from fitting_plotting_outputs.csv.

    Returns
    -------
    x_data: array
        log of the wavelength of the data in microns.
    y_data: array
        log of the flux of the data in w*m^-2
    x_model: array
        log of the wavelength of the model in microns.
    y_model: array
        log of the flux of the model in w*m^-2
    target_name: str
        targetname of source with underscores and extension removed

    """
    full_path = str(__file__.replace("plotting_seds.py", ""))
    input_file = Table.read("fitting_plotting_outputs.csv")
    grid_dusty = Table.read(
        full_path + "models/" + str(input_file["grid_name"][0]) + "_models.fits"
    )

    target_name = (target["target_name"]).replace(".csv", "").replace("_", " ")
    x_data, y_data = fitting_tools.get_data(target["data_file"])
    x_model, y_model = grid_dusty[target["index"]]
    x_model = x_model[np.where(y_model != 0)]
    if fnmatch(input_file["grid_name"][0], "grams*"):
        y_model = y_model * u.Jy
        y_model = y_model.to(
            u.W / (u.m * u.m), equivalencies=u.spectral_density(x_model * u.um)
        )
    else:
        y_model = y_model * u.W / (u.m * u.m)
    y_model = y_model[np.where(y_model != 0)] * input_file[counter]["norm"]

    # logscale
    x_model = np.log10(x_model)
    y_model = np.log10(y_model.value)
    x_data = np.log10(x_data)
    y_data = np.log10(y_data)
    return x_model, y_model, x_data, y_data, target_name


def create_fig():
    """Takes results from fitting_plotting_outputs.csv and plots SED.

    Returns
    -------
    png
        SED figure with data in blue and model in black.

    """

    full_path = str(__file__.replace("plotting_seds.py", ""))
    input_file = Table.read("fitting_plotting_outputs.csv")
    grid_dusty = Table.read(
        full_path + "models/" + str(input_file["grid_name"][0]) + "_models.fits"
    )

    # setting axes
    axislabel = "log $\lambda$ F$_{\lambda}$ (W m$^{-2}$)"
    if len(input_file) == 1:
        fig, ax1 = plt.subplots(1, 1, sharex=True, sharey=True, figsize=(8, 5))
    elif len(input_file) == 2:
        fig, axs = plt.subplots(2, 1, sharex=True, sharey=True, figsize=(8, 10))
    elif len(input_file) == 3:
        fig, axs = plt.subplots(3, 1, sharex=True, sharey=True, figsize=(8, 10))
    else:
        fig, axs = plt.subplots(
            math.ceil(len(input_file) / 3), 3, sharex=True, sharey=True, figsize=(8, 10)
        )
        axs = axs.ravel()

    for counter, target in enumerate(input_file):
        # gets data for plotting
        x_model, y_model, x_data, y_data, target_name = get_model_and_data_for_plotting(
            counter, target
        )

        # find y limits
        median = np.median(
            y_model[(x_model > np.min(x_data)) & (x_model < np.max(x_data))]
        )

        y_min = median - 2
        y_max = median + 2

        y_diff = np.median(y_data) - median

        if y_diff > 0:
            y_max = y_max + y_diff
        else:
            y_min = y_min - y_diff

        # plotting
        if len(input_file) == 1:
            ax1.set_xlim(-0.99, 2.49)
            ax1.set_ylim(y_min, y_max)
            ax1.scatter(x_data, y_data, c="blue", label="data")
            ax1.plot(
                x_model,
                y_model,
                c="k",
                linewidth=0.5,
                linestyle="--",
                zorder=2,
                label="model",
            )
            ax1.annotate(
                target_name.replace("-", r"\textendash"),
                (0.07, 0.85),
                xycoords="axes fraction",
                fontsize=14,
            )
            ax1.get_xaxis().set_tick_params(which="both", direction="in", labelsize=15)
            ax1.get_yaxis().set_tick_params(which="both", direction="in", labelsize=15)
            ax1.set_xlabel("log $\lambda$ ($\mu m$)", labelpad=10)
            ax1.set_ylabel("log $\lambda$ F$_{\lambda}$ " + "(W m$^{-2}$)", labelpad=10)
        else:
            axs[counter].set_xlim(-0.99, 2.49)
            axs[counter].set_ylim(y_min, y_max)
            axs[counter].plot(
                x_model, y_model, c="k", linewidth=0.4, linestyle="--", zorder=2
            )
            axs[counter].scatter(x_data, y_data, c="blue")
            axs[counter].annotate(
                target_name.replace("-", r"\textendash"),
                (0.7, 0.8),
                xycoords="axes fraction",
                fontsize=14,
            )
            axs[counter].get_xaxis().set_tick_params(
                which="both", direction="in", labelsize=15
            )
            axs[counter].get_yaxis().set_tick_params(
                which="both", direction="in", labelsize=15
            )
            axs[counter].set_xlabel("log $\lambda$ ($\mu m$)", labelpad=10)
            axs[counter].set_ylabel(axislabel, labelpad=10)

        # pdb.set_trace()
    plt.subplots_adjust(wspace=0, hspace=0)
    fig.savefig("output_sed.png", dpi=200, bbox_inches="tight")
    plt.close()


def single_figures():
    """Takes results from fitting_plotting_outputs.csv and plots SEDs in individual
     figures.

    Returns
    -------
    png's
        SED figures with data in blue and model in black.

    """

    full_path = str(__file__.replace("plotting_seds.py", ""))
    input_file = Table.read("fitting_plotting_outputs.csv")
    grid_dusty = Table.read(
        full_path + "models/" + str(input_file["grid_name"][0]) + "_models.fits"
    )

    for counter, target in enumerate(input_file):
        # gets data for plotting
        x_model, y_model, x_data, y_data, target_name = get_model_and_data_for_plotting(
            counter, target
        )

        # Figure plotting
        fig, ax1 = plt.subplots(1, 1, sharex=True, sharey=True, figsize=(8, 5))
        ax1.set_xlim(-0.99, 2.49)
        ax1.set_ylim(np.median(y_model) - 3, np.median(y_model) + 3)
        ax1.scatter(x_data, y_data, c="blue", label="data")
        ax1.plot(
            x_model,
            y_model,
            c="k",
            linewidth=0.5,
            linestyle="--",
            zorder=2,
            label="model",
        )
        ax1.annotate(
            target_name.replace("_", " "),
            (0.07, 0.85),
            xycoords="axes fraction",
            fontsize=14,
        )
        ax1.get_xaxis().set_tick_params(which="both", direction="in", labelsize=15)
        ax1.get_yaxis().set_tick_params(which="both", direction="in", labelsize=15)
        ax1.set_xlabel("log $\lambda$ ($\mu m$)", labelpad=10)
        ax1.set_ylabel("log $\lambda$ F$_{\lambda}$ " + "(W m$^{-2}$)", labelpad=10)
        fig.savefig(
            "output_sed_" + str(target_name) + ".png", dpi=200, bbox_inches="tight"
        )
        plt.close()


if __name__ == "__main__":
    create_fig()



# Steve Goldman, Space Telescope Science Institute, sgoldman@stsci.edu
import os
import copy
import glob
import time
import ipdb
import importlib
import numpy as np
import astropy.units as u
from fnmatch import fnmatch
from astropy.table import Table, Column
from multiprocessing import Value
from desk import config, plotting_seds, get_remote_models
from desk import parameter_ranges, fitting_tools, interpolate_dusty

importlib.reload(config)

# Non-fitting commands #########################################################
def grids():
    # Prints the model grids available for fitting.
    print("\nGrids:")
    for item in config.grids:
        print("\t" + str(item))
    print("\n")


# returns interpolate model as csv for any model grid
def get_model(grid_name, teff_new, tinner_new, tau_new):
    interpolate_dusty.interpolate(
        grid_name, float(teff_new), float(tinner_new), float(tau_new)
    )


def single_fig():
    plotting_seds.single_figures()


def fit(
    source="desk/put_target_data_here",
    distance=config.target["distance_in_kpc"],
    grid=config.fitting["model_grid"],
):
    """Fits the seds of sources with specified grid.

    Parameters
    ----------
    source : str
        Name of target in array of strings (or one string).
    distance : float
        Distance to source(s) in kiloparsecs.
    grid : str
        Name of model grid.

    """

    # set variables
    grid_type = copy.copy(grid)
    start = time.time()
    counter = Value("i", 0)
    # normalization calculation
    # solar constant = 1379 W
    # distance to sun in kpc 4.8483E-9
    full_path = str(__file__.replace("console_commands.py", ""))

    # create output files
    if fnmatch(grid_type, "grams*"):
        fitting_tools.make_output_files_grams()
    else:
        fitting_tools.make_output_files_dusty()

    # User input for models
    if grid == "carbon":
        model_grid = "Zubko-Crich-bb"
    elif grid == "oxygen":
        model_grid = "Oss-Orich-bb"
    else:
        if grid in config.grids:
            model_grid = grid
        else:
            raise ValueError(
                "\n\nUnknown grid. Please make another model selection.\n\n To see options use: desk grids\n"
            )

    # check if models exist
    csv_file, fits_file = fitting_tools.check_models(model_grid, full_path)

    # gets models
    grid_dusty = Table.read(fits_file)
    grid_outputs = Table.read(csv_file)

    # Model grid equal lengths check
    if len(grid_dusty) == len(grid_outputs):
        pass
    else:
        raise ValueError(
            "Model grid input error: mismatch in model spectra and model output"
        )

    # SED FITTING ###############################
    def add_variables(source):
        return fitting_tools.sed_fitting(
            model_grid,
            source,
            distance,
            grid_dusty,
            grid_outputs,
            counter,
            number_of_targets,
        )

    if fnmatch(source, "*.csv"):
        number_of_targets = 1
        add_variables(source)
    elif os.path.isdir(source):
        source_dir = (source + "/").replace("//", "/")
        if glob.glob(source_dir + "/*.csv"):
            files = glob.glob(source + "/" + "*.csv")
            number_of_targets = len(files)
            # with Pool(processes=cpu_count() - 1) as pool:
            #     pool.map(sed_fitting, [target_string for target_string in files])

            for target_string in files:
                add_variables(target_string)
        else:
            raise ValueError(
                "\n\n\nERROR: No .csv files in that directory. Please make another selection.\n\n"
            )
    else:
        raise ValueError(
            "\n\n\nError: Not a .csv file. Please make another selection.\n\n"
        )

    # creating figures
    if config.output["create_figure"] == "yes":
        print("\n. . . Creating SED figure . . . . . . . . . . . .")
        plotting_seds.create_fig()  # runs plotting script
    else:
        print(
            "No figure created. To automatically generate a figure change the "
            + '"create_figure" variable in the config.py script to "yes".'
        )

    if not os.path.isfile("parameter_ranges_" + config.fitting["model_grid"] + ".png"):
        print(". . . Creating parameter range figure . . . . . .")
        parameter_ranges.create_par()

    end = time.time()
    print()
    print("Time: " + str("%.2f" % float((end - start) / 60)) + " minutes")


if __name__ == "__main__":
    fit()



import argparse
import inspect
import pdb

from desk import console_commands


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--global-setting", action="store_true", help="some global thingie"
    )

    subparsers = parser.add_subparsers(dest="subparser_name", help="sub-command help")

    # create the subparsers and populate them with the correct arguments
    funcs_to_subcommand = inspect.getmembers(console_commands, inspect.isfunction)
    for name, func in funcs_to_subcommand:
        subparser = subparsers.add_parser(name, help=func.__doc__)
        for parname, arg in inspect.signature(func).parameters.items():
            # sanitized_name = parname.replace('_', '-')
            if arg.default == inspect.Signature.empty:
                subparser.add_argument(parname)
            else:
                subparser.add_argument("--" + parname, default=arg.default)

    # now actually parse the arguments
    args = parser.parse_args()

    # set the global
    console_commands.GLOBAL_SETTING = args.global_setting

    # and call the function
    for name, func in funcs_to_subcommand:
        if args.subparser_name == name:
            # make a dictionary containing the correct inputs to the function,
            # extracted from the parsed arguments
            funcargs = {}
            for parname in inspect.signature(func).parameters:
                funcargs[parname] = getattr(args, parname)
            # pdb.set_trace()
            func(**funcargs)
            break  # drop out immediately, which skips the "else" below
    else:
        assert False, "Invalid subparser! This should be impossible..."


if __name__ == "__main__":
    main()



"""
Steve Goldman
Space Telescope Science Institute
May 17, 2018
sgoldman@stsci.edu

This script is for plotting the outputs of the sed_fitting script.
"""

target = dict(distance_in_kpc=50, assumed_gas_to_dust_ratio=200, input_unit="Jy")

# model options
# Built-in options: Zubko-Crich-aringer, Oss-Orich-bb, Oss-Orich-aringer, Crystalline-20-bb, corundum-20-bb
# Padova options: J400, J1000, H11, R12, R13
# Grams options: grams-oxygen, grams-carbon

fitting = dict(
    model_grid="Zubko-Crich-bb",
    wavelength_min=0.00001,
    wavelength_max=100,
    number_of_tries=200,
)
output = dict(
    printed_output="True",
    output_unit="Wm^-2",
    # output_unit='Jy',
    create_figure="yes",
)
grids = [
    "Crystalline-20-bb",
    "H11-LMC",
    "H11-SMC",
    "J1000-LMC",
    "J1000-SMC",
    "Oss-Orich-aringer",
    "Oss-Orich-bb",
    "Zubko-Crich-aringer",
    "Zubko-Crich-bb",
    "arnold-palmer",
    "big-grain",
    "corundum-20-bb",
    "fifth-iron",
    "half-iron",
    "one-fifth-carbon",
    "grams-oxygen",
    "grams-carbon",
]



import csv
import copy
import math
import ipdb
import numpy as np
from astropy.table import Table
from desk import console_commands, config, fitting_tools


def grams_fit(
    model_grid, source, distance, grid_dusty, grid_outputs, counter, number_of_targets
):
    """Function fits astropy table data with a least squares method.

    Parameters
    ----------
    model_grid : str
        Name of model grid being used.
    source : str
        Name of the source being fit.
    distance : float
        Distance to source in kpc.
    grid_dusty : astropy table
        Table with two items in each row item 1 being an array
        with wavelength in microns and item 2 being an array with flux in w/m2.
    grid_outputs : astropy table
        Table with each row showing the output results corresponding to each row
        in grid_dusty
    counter : int
        The nth item being fit.
    number_of_targets : int
        The total number of sources to be fit.

    """

    # Initialize variables
    stat_values = []

    # gets target data
    raw_data = fitting_tools.get_data(source)

    # flux scaling from LMC to *distance* kpc
    flux_scaling_factor = 50 ** 2 / float(distance) ** 2

    scaled_grid = Table(
        (grid_dusty["col0"], np.multiply(grid_dusty["col1"], flux_scaling_factor))
    )

    # One big grid and no trials
    stat_values.append(
        [fitting_tools.trim_find_lsq(x, raw_data, [1]) for x in scaled_grid]
    )

    # def trim_find_lsq(model):
    #     # removes data outside of wavelegth range of model grid
    #     trimmed_model = fitting_tools.trim(raw_data, model)
    #
    #     # gets fluxes for corresponding wavelengths of data and models
    #     matched_model = fitting_tools.find_closest(raw_data, trimmed_model)
    #
    #     # normalize model to specified distance
    #     scaled_matched_model = matched_model * flux_scaling_factor
    #     ipdb.set_trace()
    #
    #     # fits source with least squares
    #     stats = fitting_tools.least2(raw_data, matched_model)
    #     stat_values.append(stats)

    # [trim_find_lsq(x) for x in grid_dusty]

    # obtains best fit model and model index
    stat_array = np.vstack(stat_values)
    model_index = np.argmin(stat_array)  # lowest chi square value
    target_name = (source.split("/")[-1][:15]).replace("IRAS-", "IRAS ")

    # scaling outputs
    distance_value = float(copy.copy(distance))
    luminosity = grid_outputs[model_index]["lum"] * ((distance_value / 50) ** 2)
    teff = grid_outputs[model_index]["teff"]
    tinner = int(grid_outputs[model_index]["tinner"])
    odep = grid_outputs[model_index]["odep"]
    mdot = grid_outputs[model_index]["mdot"] * (distance_value / 50)
    rin = grid_outputs[model_index]["rin"] * (distance_value / 50)

    # creates output file
    latex_array = [
        target_name,
        luminosity,
        rin,
        teff,
        tinner,
        odep,
        "%.3E" % float(mdot),
    ]

    plotting_array = [
        target_name,
        source,
        flux_scaling_factor,
        model_index,
        model_grid,
        teff,
        tinner,
        odep,
    ]
    if config.output["printed_output"] == "True":
        print()
        print()
        print(
            (
                "             Target: "
                + target_name
                + "        "
                + str(counter.value + 1)
                + "/"
                + str(number_of_targets)
            )
        )
        print("-------------------------------------------------")
        print(("Luminosity\t\t\t|\t" + str(round(luminosity))))
        print(
            (
                "Optical depth\t\t\t|\t"
                + str(round(grid_outputs[model_index]["odep"], 3))
            )
        )
        print(("Inner Radius\t\t\t|\t" + str("%.2E" % float(rin))))
        print(("Dust production rate \t\t|\t" + str("%.2E" % float(mdot))))
        print("-------------------------------------------------")
    with open("fitting_results.csv", "a") as f:
        writer = csv.writer(f, delimiter=",", lineterminator="\n")
        writer.writerow(np.array(latex_array))
        f.close()

    with open("fitting_plotting_outputs.csv", "a") as f:
        writer = csv.writer(f, delimiter=",", lineterminator="\n")
        writer.writerow(np.array(plotting_array))
        f.close()
    counter.value += 1



import fnmatch
import math
import matplotlib.pyplot as plt
import numpy as np
from astropy.table import Table


def create_par():
    counter = 0
    input_file = Table.read("fitting_plotting_outputs.csv")
    model = input_file["grid_name"][0]
    full_path = str(__file__.replace("parameter_ranges.py", ""))
    par = Table.read(full_path + "models/" + model + "_outputs.csv")

    for i in par.colnames:
        if fnmatch.fnmatch(par[i].dtype.str, "<U*"):
            par.remove_column(i)
        else:
            par.rename_column(i, i.replace("_", " "))

    if len(par.colnames) > 18:
        par = par[par.colnames[:17]]

    if len(par.colnames) < 8:
        fig, axs = plt.subplots(math.ceil(len(par.colnames)), 1, figsize=(8, 10))
        plt.subplots_adjust(wspace=0, hspace=0.5)
    else:
        fig, axs = plt.subplots(math.ceil(len(par.colnames)), 1, figsize=(5, 10))
        plt.subplots_adjust(wspace=0, hspace=1)

    axs = axs.ravel()

    axs[0].set_title("Model grid: " + model, size=20)

    for col in par.colnames:
        par_min = np.min(par[col])
        par_max = np.max(par[col])
        axs[counter].scatter(
            par[col], [0] * len(par), marker="|", alpha=0.3, c="royalblue"
        )
        range_min = par_min - ((par_max - par_min) * 0.1)
        range_max = par_max + (par_max - par_min) * 0.1
        if range_min != range_max:
            axs[counter].set_xlim(range_min, range_max)
        elif par_min == 0 and par_max == 0:
            axs[counter].set_xlim(-0.9, 0.9)
        else:
            axs[counter].set_xlim(range_min * 0.6, range_min * 1.4)
        # print(str(par_min) + ' : ' + str(par_max))
        axs[counter].set_ylabel(col)
        axs[counter].set_yticklabels([])
        axs[counter].set_yticks([])
        counter += 1
    fig.savefig("parameter_ranges_" + model + ".png", dpi=200, bbox_inches="tight")


if __name__ == "__main__":
    create_par()



import os
import pdb
import ipdb
import shutil
import sys
import tarfile
import urllib
from fnmatch import fnmatch

import numpy as np
from astropy.table import vstack, Table, Column
from tqdm import tqdm


def get_models(model_grid_name):
    def reporthook(blocknum, blocksize, totalsize):
        readsofar = blocknum * blocksize
        if totalsize > 0:
            percent = readsofar * 1e2 / totalsize
            s = "\r%5.0f%% %*d / %d KB" % (
                percent,
                len(str(totalsize)),
                readsofar / 1e3,
                totalsize / 1e3,
            )
            sys.stderr.write(s)
            if readsofar >= totalsize:  # near the end
                sys.stderr.write("\n")
        else:  # total size is unknown
            sys.stderr.write("read %d\n" % (readsofar,))

    full_path = str(__file__.replace("/get_remote_models.py", ""))
    grids = [
        "Crystalline-20-bb",
        "H11-SMC",
        "H11-LMC",
        "J1000-SMC",
        "J1000-LMC",
        "Oss-Orich-aringer",
        "Oss-Orich-bb",
        "Zubko-Crich-aringer",
        "Zubko-Crich-bb",
        "arnold-palmer",
        "big-grain",
        "corundum-20-bb",
        "fifth-iron",
        "half-iron",
        "one-fifth-carbon",
        "grams-oxygen",
        "grams-carbon",
    ]
    grid_csv_link = [
        "https://stsci.box.com/shared/static/jxtabj6h5zi7a8ggb0mbtag3v6b4b9i7.csv",
        "https://stsci.box.com/shared/static/z0hnvo0hz0qxcwlcvbmf01avdjlhewah.csv",
        "https://stsci.box.com/shared/static/j7f01dwbsbluq4htdwlsx3v0jknye883.csv",
        "https://stsci.box.com/shared/static/o6dp1cyj86jw2ifuuco983kgb2uf16sf.csv",
        "https://stsci.box.com/shared/static/fuhrcgo4vh4m7ca121p1f0vuss4vszcz.csv",
        "https://stsci.box.com/shared/static/he92pt1yov18x6ska5f4f4q94gal6tr5.csv",
        "https://stsci.box.com/shared/static/pnljezbu8c7rcyorb27yh6hylk5wy2fs.csv",
        "https://stsci.box.com/shared/static/x9fg8lacbbiye591tz723ilcqgcomyjh.csv",
        "https://stsci.box.com/shared/static/4b9rc9zl110khexeuo8thb8r58mrl73g.csv",
        "https://stsci.box.com/shared/static/4gxmapkhml7lnxsiu9dzto8a7zwnmzd5.csv",
        "https://stsci.box.com/shared/static/kfaq7jqt4t69vitf1cifk5gurx7lnogn.csv",
        "https://stsci.box.com/shared/static/xf6rrfnp1jdin94iwhwb47cxoplu3aux.csv",
        "https://stsci.box.com/shared/static/5zqshpzw748n0doykm7d5cxnlvhue4kk.csv",
        "https://stsci.box.com/shared/static/x7328lqy8fyg19wswqwq4md4gm398ewi.csv",
        "https://stsci.box.com/shared/static/8j73qttzj10eu5sct0ds0835uxbxu3s8.csv",
        "https://stsci.box.com/shared/static/ldocm4yto4vtxfcg21o23a8uilb3a3do.csv",
        "https://stsci.box.com/shared/static/iy6dsqrnhi96ecnlsozo1ljr5g8b1jwx.csv",
    ]
    grid_fits_link = [
        "https://stsci.box.com/shared/static/1suiv6nqbc7yoqutli2q9q98ste31o1y.fits",
        "https://stsci.box.com/shared/static/bq384o6m1xm41bkx7s8oloohqkd87hy8.fits",
        "https://stsci.box.com/shared/static/faekj3usdme9go3lmynga3uk0oln8v3k.fits",
        "https://stsci.box.com/shared/static/w0e3f1a0r886u17vjosk8lvqrnvnw6ce.fits",
        "https://stsci.box.com/shared/static/e0nvigaed722a67043rvmqhjzkonttp4.fits",
        "https://stsci.box.com/shared/static/60sns7ua91ixuz4rddul2ufrmc0pu5yo.fits",
        "https://stsci.box.com/shared/static/hr113z7lgvggyybh9eygi6nj69gu1gzv.fits",
        "https://stsci.box.com/shared/static/hvgfnug5xxcepcz083cnrizsukgddttw.fits",
        "https://stsci.box.com/shared/static/c5gdw6o3b96kaphe07y459zd7ba6590f.fits",
        "https://stsci.box.com/shared/static/b78ks7cjzenagoznqqrdfoyh6bonw2sy.fits",
        "https://stsci.box.com/shared/static/lw6gp2s8surgbbtmcnyev673faxqg9h6.fits",
        "https://stsci.box.com/shared/static/yq6pbvepyt8a420ealg6i111xc7hcty5.fits",
        "https://stsci.box.com/shared/static/qjuq780xr0ihj9p909wmijwkpclu2njd.fits",
        "https://stsci.box.com/shared/static/ht6edjsrupwhytwuwa9hjg53dof2zzy0.fits",
        "https://stsci.box.com/shared/static/n1ng9f4s8s7ps0ah8vp24gk0fzsxp0aj.fits",
        "https://stsci.box.com/shared/static/l1qayndbz40lwewvlpg1qxu22rs9f9xu.fits",
        "https://stsci.box.com/shared/static/jho46vam6d0k9jwg78gxx4eh7emwx4oz.fits",
    ]
    if any(ext in model_grid_name for ext in grids):
        match_index = [i for i, item in enumerate(grids) if model_grid_name == item][0]
        url_csv = grid_csv_link[match_index]
        url_fits = grid_fits_link[match_index]
    else:
        raise ValueError(
            "ERROR: Model name not an option. \nCurrent downloadable options: \n \t Zubko-Crich-aringer \n \t Oss-Orich-bb \n \t Oss-Orich-aringer \n \t Crystalline-20-bb \n \t corundum-20-bb \n \t arnold-palmer \n \t big-grains \n \t fifth-iron \n \t one-fifth-carbon"
        )

    # \n Padova options: J400, J1000, H11, R12, R13'

    # Download files
    print(". . . Downloading model: " + model_grid_name + " . . .")
    urllib.request.urlretrieve(
        url_csv, full_path + "/models/" + model_grid_name + "_outputs.csv"
    )
    urllib.request.urlretrieve(
        url_fits, full_path + "/models/" + model_grid_name + "_models.fits", reporthook
    )
    print("Download Complete!")


if __name__ == "__main__":
    get_models()



import csv
import copy
import math
import numpy as np
from desk import console_commands, config, fitting_tools


def dusty_fit(
    model_grid, source, distance, grid_dusty, grid_outputs, counter, number_of_targets
):
    """Function fits astropy table data with a least squares method.

    Parameters
    ----------
    model_grid : str
        Name of model grid being used.
    source : str
        Name of the source being fit.
    distance : float
        Distance to source in kpc.
    grid_dusty : astropy table
        Table with two items in each row item 1 being an array
        with wavelength in microns and item 2 being an array with flux in w/m2.
    grid_outputs : astropy table
        Table with each row showing the output results corresponding to each row
        in grid_dusty
    counter : int
        The nth item being fit.
    number_of_targets : int
        The total number of sources to be fit.

    """

    # Initialize variables
    stat_values = []

    # gets target data
    raw_data = fitting_tools.get_data(source)

    trials = fitting_tools.create_trials(raw_data[1])

    stat_values.append(
        [fitting_tools.trim_find_lsq(x, raw_data, trials) for x in grid_dusty]
    )

    # obtains best fit model and model index
    stat_array = np.vstack(stat_values)
    argmin = np.argmin(stat_array)  # lowest chi square value
    # total with e.g. len 300 and grid with e.g. len 100, yields grid index
    model_index = argmin // stat_array.shape[1]
    trial_index = argmin % stat_array.shape[1]
    target_name = (source.split("/")[-1][:15]).replace("IRAS-", "IRAS ")

    # normalizes output values by the set distance
    distance_value = float(copy.copy(distance))
    distance_norm = math.log10(((float(distance) / 4.8482e-9) ** 2) / 1379)
    luminosity = int(
        np.power(10.0, distance_norm - math.log10(trials[trial_index]) * -1)
    )
    scaled_vexp = (
        float(grid_outputs[model_index]["vexp"]) * (luminosity / 10000) ** 0.25
    )
    scaled_mdot = (
        grid_outputs[model_index]["mdot"]
        * ((luminosity / 10000) ** 0.75)
        * (config.target["assumed_gas_to_dust_ratio"] / 200) ** 0.5
    )

    teff = int(grid_outputs[model_index]["teff"])
    tinner = int(grid_outputs[model_index]["tinner"])
    odep = grid_outputs[model_index]["odep"]

    # creates results file
    latex_array = [
        target_name,
        luminosity,
        np.round(scaled_vexp, 1),
        teff,
        tinner,
        odep,
        "%.3E" % float(scaled_mdot),
    ]

    # creates file for creating figure
    plotting_array = [
        target_name,
        source,
        trials[trial_index],
        model_index,
        model_grid,
        teff,
        tinner,
        odep,
    ]

    # printed output
    if config.output["printed_output"] == "True":
        print()
        print()
        print(
            (
                "             Target: "
                + target_name
                + "        "
                + str(counter.value + 1)
                + "/"
                + str(number_of_targets)
            )
        )
        print("-------------------------------------------------")
        print(("Luminosity\t\t\t|\t" + str(round(luminosity))))
        print(
            (
                "Optical depth\t\t\t|\t"
                + str(round(grid_outputs[model_index]["odep"], 3))
            )
        )
        print(("Expansion velocity (scaled)\t|\t" + str(round(scaled_vexp, 2))))
        print(("Gas mass loss (scaled)\t\t|\t" + str("%.2E" % float(scaled_mdot))))
        print("-------------------------------------------------")

    # saves files
    with open("fitting_results.csv", "a") as f:
        writer = csv.writer(f, delimiter=",", lineterminator="\n")
        writer.writerow(np.array(latex_array))
        f.close()

    with open("fitting_plotting_outputs.csv", "a") as f:
        writer = csv.writer(f, delimiter=",", lineterminator="\n")
        writer.writerow(np.array(plotting_array))
        f.close()
    counter.value += 1



import os, sys, pdb
import numpy as np
from astropy.table import Table
from scipy.interpolate import RegularGridInterpolator
from desk import fitting_tools

# Example
# grid_name = "Oss-Orich-bb"
# teff_new = 4010
# tinner_new = 900
# tau_new = 0.146


def interpolate(grid_name, teff_new, tinner_new, tau_new):
    # checks if grid files available
    full_path = str(__file__.replace("interpolate_dusty.py", ""))
    fitting_tools.check_models(grid_name, full_path)
    try:
        output_array = Table.read(
            full_path + "models/" + grid_name + "_outputs.csv", format="csv"
        )
        grid_dusty = Table.read(full_path + "models/" + grid_name + "_models.fits")
        waves = grid_dusty[0][0]
    except:
        raise Exception(
            "Could not find model grid. Please double check input or try another grid. To see available grids use the command: desk grids"
        )
        sys.exit()

    # create interpolator
    tau = np.unique(output_array["odep"])
    teff = np.unique(output_array["teff"])
    tinner = np.unique(output_array["tinner"])

    # checks if new values within range
    if (
        (teff.min() <= teff_new <= teff.max())
        & (tinner.min() <= tinner_new <= tinner.max())
        & (tau.min() <= tau_new <= tau.max())
    ):
        pass
    else:
        raise Exception(
            "Interpolation values outside of range. Try values in range:"
            + "\n\n\tTeff:\t"
            + str(teff.min())
            + " - "
            + str(teff.max())
            + "\n\tTinner:\t "
            + str(tinner.min())
            + " - "
            + str(tinner.max())
            + "\n\tTau:\t "
            + str(tau.min())
            + " - "
            + str(tau.max())
            + "\n"
        )
        sys.exit()

    # if model already exists
    if (teff_new in teff) & (tinner_new in tinner) & (tau_new in tau):
        print("Model exists:")
        ind = np.where(
            (output_array["teff"] == teff_new)
            & (output_array["tinner"] == tinner_new)
            & (output_array["odep"] == tau_new)
        )[0][0]
        interp_dusty = Table((grid_dusty[ind]["col0"], grid_dusty[ind]["col1"]))
        expansion_velocity = output_array["vexp"][ind]
        mass_loss_rate = output_array["mdot"][ind]

    else:
        print("Interpolating model:")
        array = np.zeros((len(teff), len(tinner), len(tau), len(waves)))
        mass_loss_array = np.zeros((len(teff), len(tinner), len(tau)))
        expansion_velocity_array = np.zeros((len(teff), len(tinner), len(tau)))

        for i in range(0, len(teff)):
            for j in range(0, len(tinner)):
                for k in range(0, len(tau)):
                    mc_index = np.where(
                        (teff[i] == output_array["teff"])
                        & (tinner[j] == output_array["tinner"])
                        & (tau[k] == output_array["odep"])
                    )
                    if np.any(mc_index):
                        array[i][j][k] = grid_dusty[mc_index[0][0]][1]
                        mass_loss_array[i][j][k] = output_array["mdot"][mc_index]
                        expansion_velocity_array[i][j][k] = output_array["vexp"][
                            mc_index
                        ]
        interpolator = RegularGridInterpolator((teff, tinner, tau, waves), array)
        mdot_interpolator = RegularGridInterpolator(
            (teff, tinner, tau), mass_loss_array
        )
        vexp_interpolator = RegularGridInterpolator(
            (teff, tinner, tau), expansion_velocity_array
        )

        interp_array = []
        interp_dusty = []
        y_new = []

        for wavelength in waves:
            y_new.append(interpolator([teff_new, tinner_new, tau_new, wavelength])[0])
        interp_dusty = Table([waves, y_new])
        mass_loss_rate = mdot_interpolator([teff_new, tinner_new, tau_new])[0]
        expansion_velocity = vexp_interpolator([teff_new, tinner_new, tau_new])[0]

    print("\nExpansion velocity = " + "%.2f" % float(expansion_velocity) + " km/s")
    print("Gas mass-loss rate = " + "%.3E" % float(mass_loss_rate) + " Msun/yr\n")

    interp_dusty.write(
        grid_name
        + "_"
        + str(teff_new)
        + "_"
        + str(tinner_new)
        + "_"
        + str(tau_new)
        + ".csv",
        overwrite=True,
    )


if __name__ == "__main__":
    interpolate()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

setup(
    author="Steven R. Goldman",
    author_email="sgoldman@stsci.edu",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    description="The DESK is an SED-fitting python scripts for fitting data from evolved stars",
    entry_points={"console_scripts": ["desk = desk.main:main"]},
    install_requires=["Click>=6.0", "tqdm", "scipy", "ipdb"],
    license="BSD license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="Dusty_Evolved_Star_Kit",
    name="desk",
    packages=find_packages(include=["desk"]),
    package_data={"project": ["desk/models/*", "desk/put_target_data_here/*"]},
    setup_requires=["pytest-runner"],
    test_suite="tests",
    tests_require="pytest",
    url="https://github.com/s-goldman/Dusty_Evolved_Star_Kit",
    version="1.6.14",
    zip_safe=False,
)



# Licensed under a 3-clause BSD style license - see LICENSE.rst
"""
This packages contains python packages that are bundled with the package but
are external to it, and hence are developed in a separate source tree. Note
that this package is distinct from the /cextern directory of the source code
distribution, as that directory only contains C extension code.
"""

# -*- coding: utf-8 -*-

"""Top-level package for Dusty-Evolved-Star-Kit."""

__author__ = """Steven R. Goldman"""
__email__ = "sgoldman@stsci.edu"
__version__ = "1.4.7"
